@@ -1,10 +1,10 @@
-/* ---------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
  *
  * Giada - Your Hardcore Loopmachine
  *
  * gd_config
  *
- * ---------------------------------------------------------------------
+ * -----------------------------------------------------------------------------
  *
  * Copyright (C) 2010-2015 Giovanni A. Zuliani | Monocasual
  *
@@ -24,14 +24,16 @@
  * along with Giada - Your Hardcore Loopmachine. If not, see
  * <http://www.gnu.org/licenses/>.
  *
- * ------------------------------------------------------------------ */
+ * -------------------------------------------------------------------------- */
 
 
 #include "gd_config.h"
 #include "gd_keyGrabber.h"
 #include "gd_devInfo.h"
+#include "gd_browser.h"
 #include "ge_mixed.h"
 #include "conf.h"
+#include "midiMapConf.h"
 #include "log.h"
 #include "gui_utils.h"
 #include "patch.h"
@@ -42,9 +44,10 @@
 extern Patch G_Patch;
 extern Conf	 G_Conf;
 extern bool  G_audio_status;
+extern MidiMapConf G_MidiMap;
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 gTabMisc::gTabMisc(int X, int Y, int W, int H)
@@ -74,7 +77,7 @@ gTabMisc::gTabMisc(int X, int Y, int W, int H)
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabMisc::save()
@@ -93,9 +96,9 @@ void gTabMisc::save()
 }
 
 
-/* ------------------------------------------------------------------ */
-/* ------------------------------------------------------------------ */
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
 
 
 gTabAudio::gTabAudio(int X, int Y, int W, int H)
@@ -216,7 +219,7 @@ gTabAudio::gTabAudio(int X, int Y, int W, int H)
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabAudio::cb_deactivate_sounddev(Fl_Widget *w, void *p) { ((gTabAudio*)p)->__cb_deactivate_sounddev(); }
@@ -226,7 +229,7 @@ void gTabAudio::cb_showInputInfo(Fl_Widget *w, void *p)       { ((gTabAudio*)p)-
 void gTabAudio::cb_showOutputInfo(Fl_Widget *w, void *p)      { ((gTabAudio*)p)->__cb_showOutputInfo(); }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabAudio::__cb_fetchInChans()
@@ -236,7 +239,7 @@ void gTabAudio::__cb_fetchInChans()
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabAudio::__cb_fetchOutChans()
@@ -246,7 +249,7 @@ void gTabAudio::__cb_fetchOutChans()
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabAudio::__cb_showInputInfo()
@@ -256,7 +259,7 @@ void gTabAudio::__cb_showInputInfo()
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabAudio::__cb_showOutputInfo()
@@ -266,7 +269,7 @@ void gTabAudio::__cb_showOutputInfo()
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabAudio::__cb_deactivate_sounddev()
@@ -309,7 +312,7 @@ void gTabAudio::__cb_deactivate_sounddev()
 	}
 }
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabAudio::fetchInChans(int menuItem)
@@ -346,7 +349,7 @@ void gTabAudio::fetchInChans(int menuItem)
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabAudio::fetchOutChans(int menuItem)
@@ -370,7 +373,7 @@ void gTabAudio::fetchOutChans(int menuItem)
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 int gTabAudio::findMenuDevice(gChoice *m, int device)
@@ -394,7 +397,7 @@ int gTabAudio::findMenuDevice(gChoice *m, int device)
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabAudio::fetchSoundDevs()
@@ -462,7 +465,7 @@ void gTabAudio::fetchSoundDevs()
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabAudio::save()
@@ -512,9 +515,9 @@ void gTabAudio::save()
 }
 
 
-/* ------------------------------------------------------------------ */
-/* ------------------------------------------------------------------ */
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
 
 
 gTabMidi::gTabMidi(int X, int Y, int W, int H)
@@ -525,7 +528,8 @@ gTabMidi::gTabMidi(int X, int Y, int W, int H)
 	portOut	  = new gChoice(x()+92, system->y()+system->h()+8, 253, 20, "Output port");
 	portIn	  = new gChoice(x()+92, portOut->y()+portOut->h()+8, 253, 20, "Input port");
 	noNoteOff = new gCheck (x()+92, portIn->y()+portIn->h()+8, 253, 20, "Device does not send NoteOff");
-	sync	    = new gChoice(x()+92, noNoteOff->y()+noNoteOff->h(), 253, 20, "Sync");
+	midiMap	  = new gChoice(x()+92, noNoteOff->y()+noNoteOff->h(), 253, 20, "Output Midi Map");
+	sync	    = new gChoice(x()+92, midiMap->y()+midiMap->h()+8, 253, 20, "Sync");
 	new gBox(x(), sync->y()+sync->h()+8, w(), h()-125, "Restart Giada for the changes to take effect.");
 	end();
 
@@ -536,6 +540,7 @@ gTabMidi::gTabMidi(int X, int Y, int W, int H)
 	fetchSystems();
 	fetchOutPorts();
 	fetchInPorts();
+	fetchMidiMaps();
 
 	noNoteOff->value(G_Conf.noNoteOff);
 
@@ -553,7 +558,7 @@ gTabMidi::gTabMidi(int X, int Y, int W, int H)
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabMidi::fetchOutPorts() {
@@ -579,7 +584,7 @@ void gTabMidi::fetchOutPorts() {
 	}
 }
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabMidi::fetchInPorts()
@@ -606,7 +611,27 @@ void gTabMidi::fetchInPorts()
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
+
+
+void gTabMidi::fetchMidiMaps()
+{
+	if (G_MidiMap.maps.size == 0) {
+		midiMap->add("(no MIDI maps available)");
+		midiMap->value(0);
+		midiMap->deactivate();
+		return;
+	}
+	for (unsigned i=0; i<G_MidiMap.maps.size; i++) {
+		const char *imap = G_MidiMap.maps.at(i).c_str();
+		midiMap->add(imap);
+		if (strcmp(G_Conf.midiMapPath, imap) == 0)
+			midiMap->value(i);
+	}
+}
+
+
+/* -------------------------------------------------------------------------- */
 
 
 void gTabMidi::save()
@@ -617,8 +642,6 @@ void gTabMidi::save()
 		G_Conf.midiSystem = RtMidi::UNIX_JACK;
 	else if (!strcmp("Multimedia MIDI", system->text(system->value())))
 		G_Conf.midiSystem = RtMidi::WINDOWS_MM;
-	//else if (!strcmp("Kernel Streaming MIDI", system->text(system->value())))
-	//	G_Conf.midiSystem = RtMidi::WINDOWS_KS;
 	else if (!strcmp("OSX Core MIDI", system->text(system->value())))
 		G_Conf.midiSystem = RtMidi::MACOSX_CORE;
 
@@ -627,6 +650,8 @@ void gTabMidi::save()
 
 	G_Conf.noNoteOff   = noNoteOff->value();
 
+	G_Conf.setPath(G_Conf.midiMapPath, midiMap->text(midiMap->value()));
+
 	if      (sync->value() == 0)
 		G_Conf.midiSync = MIDI_SYNC_NONE;
 	else if (sync->value() == 1)
@@ -636,7 +661,7 @@ void gTabMidi::save()
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabMidi::fetchSystems()
@@ -652,8 +677,6 @@ void gTabMidi::fetchSystems()
 
 	if (kernelMidi::hasAPI(RtMidi::WINDOWS_MM))
 		system->add("Multimedia MIDI");
-	//if (kernelMidi::hasAPI(RtMidi::WINDOWS_KS))
-	//	system->add("Kernel Streaming MIDI");
 
 #elif defined (__APPLE__)
 
@@ -665,20 +688,19 @@ void gTabMidi::fetchSystems()
 		case RtMidi::LINUX_ALSA:  system->show("ALSA"); break;
 		case RtMidi::UNIX_JACK:   system->show("Jack"); break;
 		case RtMidi::WINDOWS_MM:  system->show("Multimedia MIDI"); break;
-		//case RtMidi::WINDOWS_KS:  system->show("Kernel Streaming MIDI"); break;
 		case RtMidi::MACOSX_CORE: system->show("OSX Core MIDI"); break;
 		default: system->value(0); break;
 	}
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabMidi::cb_changeSystem(Fl_Widget *w, void *p) { ((gTabMidi*)p)->__cb_changeSystem(); }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabMidi::__cb_changeSystem()
@@ -702,9 +724,9 @@ void gTabMidi::__cb_changeSystem()
 }
 
 
-/* ------------------------------------------------------------------ */
-/* ------------------------------------------------------------------ */
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
 
 
 gTabBehaviors::gTabBehaviors(int X, int Y, int W, int H)
@@ -739,13 +761,13 @@ gTabBehaviors::gTabBehaviors(int X, int Y, int W, int H)
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabBehaviors::cb_radio_mutex(Fl_Widget *w, void *p) { ((gTabBehaviors*)p)->__cb_radio_mutex(w); }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabBehaviors::__cb_radio_mutex(Fl_Widget *w)
@@ -754,7 +776,7 @@ void gTabBehaviors::__cb_radio_mutex(Fl_Widget *w)
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gTabBehaviors::save()
@@ -765,9 +787,9 @@ void gTabBehaviors::save()
 }
 
 
-/* ------------------------------------------------------------------ */
-/* ------------------------------------------------------------------ */
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
 
 
 gdConfig::gdConfig(int w, int h) : gWindow(w, h, "Configuration")
@@ -802,7 +824,7 @@ gdConfig::gdConfig(int w, int h) : gWindow(w, h, "Configuration")
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 gdConfig::~gdConfig()
@@ -812,14 +834,14 @@ gdConfig::~gdConfig()
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gdConfig::cb_save_config(Fl_Widget *w, void *p) { ((gdConfig*)p)->__cb_save_config(); }
 void gdConfig::cb_cancel     (Fl_Widget *w, void *p) { ((gdConfig*)p)->__cb_cancel(); }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gdConfig::__cb_save_config()
@@ -832,7 +854,7 @@ void gdConfig::__cb_save_config()
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 void gdConfig::__cb_cancel()
