@@ -35,7 +35,7 @@
 #include "gd_warnings.h"
 #include "gd_bpmInput.h"
 #include "gd_beatsInput.h"
-#include "gd_midiGrabber.h"
+#include "gd_midiInput.h"
 #include "gg_keyboard.h"
 #include "gd_about.h"
 #include "gd_config.h"
@@ -413,7 +413,7 @@ void gMenu::__cb_edit()
 		return;
 	}
 	if (strcmp(m->label(), "Setup global MIDI input...") == 0) {
-		gu_openSubWindow(mainWin, new gdMidiGrabberMaster(), 0);
+		gu_openSubWindow(mainWin, new gdMidiInputMaster(), 0);
 		return;
 	}
 }
