@@ -1,10 +1,10 @@
-/* ---------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
  *
  * Giada - Your Hardcore Loopmachine
  *
  * channel
  *
- * ---------------------------------------------------------------------
+ * -----------------------------------------------------------------------------
  *
  * Copyright (C) 2010-2015 Giovanni A. Zuliani | Monocasual
  *
@@ -24,7 +24,7 @@
  * along with Giada - Your Hardcore Loopmachine. If not, see
  * <http://www.gnu.org/licenses/>.
  *
- * ------------------------------------------------------------------ */
+ * -------------------------------------------------------------------------- */
 
 
 #include <math.h>
@@ -35,6 +35,7 @@
 #include "pluginHost.h"
 #include "waveFx.h"
 #include "mixerHandler.h"
+#include "kernelMidi.h"
 #include "log.h"
 
 
@@ -72,10 +73,11 @@ SampleChannel::SampleChannel(int bufferSize)
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-SampleChannel::~SampleChannel() {
+SampleChannel::~SampleChannel()
+{
 	if (wave)
 		delete wave;
 	src_delete(rsmp_state);
@@ -83,11 +85,11 @@ SampleChannel::~SampleChannel() {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::clear() {
-
+void SampleChannel::clear()
+{
 	/** TODO - these memsets can be done only if status PLAY (if below),
 	 * but it would require extra clearPChan calls when samples stop */
 
@@ -104,11 +106,11 @@ void SampleChannel::clear() {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::calcVolumeEnv(int frame) {
-
+void SampleChannel::calcVolumeEnv(int frame)
+{
 	/* method: check this frame && next frame, then calculate delta */
 
 	recorder::action *a0 = NULL;
@@ -137,21 +139,24 @@ void SampleChannel::calcVolumeEnv(int frame) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::hardStop(int frame) {
+void SampleChannel::hardStop(int frame)
+{
 	if (frame != 0)        // clear data in range [frame, bufferSize-1]
 		clearChan(vChan, frame);
 	status = STATUS_OFF;
+	sendMidiLplay();
 	reset(frame);
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::onBar(int frame) {
+void SampleChannel::onBar(int frame)
+{
 	///if (mode == LOOP_REPEAT && status == STATUS_PLAY)
 	///	//setXFade(frame);
 	///	reset(frame);
@@ -166,41 +171,45 @@ void SampleChannel::onBar(int frame) {
 		if (status == STATUS_WAIT) {
 			status  = STATUS_PLAY;
 			tracker = fillChan(vChan, tracker, frame);
+			sendMidiLplay();
 		}
 	}
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-int SampleChannel::save(const char *path) {
+int SampleChannel::save(const char *path)
+{
 	return wave->writeData(path);
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::setBegin(unsigned v) {
+void SampleChannel::setBegin(unsigned v)
+{
 	begin   = v;
 	tracker = begin;
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::setEnd(unsigned v) {
+void SampleChannel::setEnd(unsigned v)
+{
 	end = v;
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::setPitch(float v) {
-
+void SampleChannel::setPitch(float v)
+{
 	pitch = v;
 	rsmp_data.src_ratio = 1/pitch;
 
@@ -211,11 +220,11 @@ void SampleChannel::setPitch(float v) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::rewind() {
-
+void SampleChannel::rewind()
+{
 	/* rewind LOOP_ANY or SINGLE_ANY only if it's in read-record-mode */
 
 	if (wave != NULL) {
@@ -225,11 +234,11 @@ void SampleChannel::rewind() {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::parseAction(recorder::action *a, int localFrame, int globalFrame) {
-
+void SampleChannel::parseAction(recorder::action *a, int localFrame, int globalFrame)
+{
 	if (readActions == false)
 		return;
 
@@ -259,11 +268,11 @@ void SampleChannel::parseAction(recorder::action *a, int localFrame, int globalF
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::sum(int frame, bool running) {
-
+void SampleChannel::sum(int frame, bool running)
+{
 	if (wave == NULL || status & ~(STATUS_PLAY | STATUS_ENDING))
 		return;
 
@@ -349,14 +358,17 @@ void SampleChannel::sum(int frame, bool running) {
 			 (mode & LOOP_ANY && !running))     // stop loops when the seq is off
 		{
 			status = STATUS_OFF;
+			sendMidiLplay();
 		}
 
 		/* temporary stop LOOP_ONCE not in ENDING status, otherwise they
 		 * would return in wait, losing the ENDING status */
 
 		//if (mode == LOOP_ONCE && status != STATUS_ENDING)
-		if ((mode & (LOOP_ONCE | LOOP_ONCE_BAR)) && status != STATUS_ENDING)
+		if ((mode & (LOOP_ONCE | LOOP_ONCE_BAR)) && status != STATUS_ENDING) {
 			status = STATUS_WAIT;
+			sendMidiLplay();
+		}
 
 		/* check for end of samples. SINGLE_ENDLESS runs forever unless
 		 * it's in ENDING mode. */
@@ -366,11 +378,11 @@ void SampleChannel::sum(int frame, bool running) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::onZero(int frame) {
-
+void SampleChannel::onZero(int frame)
+{
 	if (wave == NULL)
 		return;
 
@@ -395,6 +407,7 @@ void SampleChannel::onZero(int frame) {
 
 	if (status == STATUS_WAIT) { /// FIXME - should be inside previous if!
 		status  = STATUS_PLAY;
+		sendMidiLplay();
 		tracker = fillChan(vChan, tracker, frame);
 	}
 
@@ -410,11 +423,11 @@ void SampleChannel::onZero(int frame) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::quantize(int index, int localFrame, int globalFrame) {
-
+void SampleChannel::quantize(int index, int localFrame, int globalFrame)
+{
 	/* skip if LOOP_ANY or not in quantizer-wait mode */
 
 	if ((mode & LOOP_ANY) || !qWait)
@@ -425,6 +438,7 @@ void SampleChannel::quantize(int index, int localFrame, int globalFrame) {
 
 	if (status == STATUS_OFF) {
 		status  = STATUS_PLAY;
+		sendMidiLplay();
 		qWait   = false;
 		tracker = fillChan(vChan, tracker, localFrame); /// FIXME: ???
 	}
@@ -444,10 +458,11 @@ void SampleChannel::quantize(int index, int localFrame, int globalFrame) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-int SampleChannel::getPosition() {
+int SampleChannel::getPosition()
+{
 	if (status & ~(STATUS_EMPTY | STATUS_MISSING | STATUS_OFF)) // if is not (...)
 		return tracker - begin;
 	else
@@ -455,11 +470,11 @@ int SampleChannel::getPosition() {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::setMute(bool internal) {
-
+void SampleChannel::setMute(bool internal)
+{
 	if (internal) {
 
 		/* global mute is on? don't waste time with fadeout, just mute it
@@ -491,13 +506,16 @@ void SampleChannel::setMute(bool internal) {
 				mute = true;
 		}
 	}
+
+	sendMidiLmute();
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::unsetMute(bool internal) {
+void SampleChannel::unsetMute(bool internal)
+{
 	if (internal) {
 		if (mute)
 			mute_i = false;
@@ -518,13 +536,16 @@ void SampleChannel::unsetMute(bool internal) {
 				mute = false;
 		}
 	}
+
+	sendMidiLmute();
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::calcFadeoutStep() {
+void SampleChannel::calcFadeoutStep()
+{
 	if (end - tracker < (1 / DEFAULT_FADEOUT_STEP) * 2)
 		fadeoutStep = ceil((end - tracker) / volume) * 2; /// or volume_i ???
 	else
@@ -532,10 +553,11 @@ void SampleChannel::calcFadeoutStep() {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::setReadActions(bool v) {
+void SampleChannel::setReadActions(bool v)
+{
 	if (v)
 		readActions = true;
 	else {
@@ -546,10 +568,11 @@ void SampleChannel::setReadActions(bool v) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::setFadeIn(bool internal) {
+void SampleChannel::setFadeIn(bool internal)
+{
 	if (internal) mute_i = false;  // remove mute before fading in
 	else          mute   = false;
 	fadeinOn  = true;
@@ -557,10 +580,11 @@ void SampleChannel::setFadeIn(bool internal) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::setFadeOut(int actionPostFadeout) {
+void SampleChannel::setFadeOut(int actionPostFadeout)
+{
 	calcFadeoutStep();
 	fadeoutOn   = true;
 	fadeoutVol  = 1.0f;
@@ -569,11 +593,11 @@ void SampleChannel::setFadeOut(int actionPostFadeout) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::setXFade(int frame) {
-
+void SampleChannel::setXFade(int frame)
+{
 	gLog("[xFade] frame=%d tracker=%d\n", frame, tracker);
 
 	calcFadeoutStep();
@@ -585,7 +609,7 @@ void SampleChannel::setXFade(int frame) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
 /* on reset, if frame > 0 and in play, fill again pChan to create
@@ -596,7 +620,8 @@ void SampleChannel::setXFade(int frame) {
  *
  * */
 
-void SampleChannel::reset(int frame) {
+void SampleChannel::reset(int frame)
+{
 	//fadeoutTracker = tracker;   // store old frame number for xfade
 	tracker = begin;
 	mute_i  = false;
@@ -605,35 +630,39 @@ void SampleChannel::reset(int frame) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::empty() {
+void SampleChannel::empty()
+{
 	status = STATUS_OFF;
 	if (wave) {
 		delete wave;
 		wave = NULL;
 	}
 	status = STATUS_EMPTY;
+	sendMidiLplay();
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::pushWave(Wave *w) {
+void SampleChannel::pushWave(Wave *w)
+{
 	wave   = w;
 	status = STATUS_OFF;
+	sendMidiLplay();
 	begin  = 0;
 	end    = wave->size;
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-bool SampleChannel::allocEmpty(int frames, int takeId) {
-
+bool SampleChannel::allocEmpty(int frames, int takeId)
+{
 	Wave *w = new Wave();
 	if (!w->allocEmpty(frames))
 		return false;
@@ -648,15 +677,17 @@ bool SampleChannel::allocEmpty(int frames, int takeId) {
 	begin       = 0;
 	end         = wave->size;
 
+	sendMidiLplay();
+
 	return true;
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::process(float *buffer) {
-
+void SampleChannel::process(float *buffer)
+{
 #ifdef WITH_VST
 	G_PluginHost.processStack(vChan, PluginHost::CHANNEL, this);
 #endif
@@ -668,10 +699,11 @@ void SampleChannel::process(float *buffer) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::kill(int frame) {
+void SampleChannel::kill(int frame)
+{
 	if (wave != NULL && status != STATUS_OFF) {
 		if (mute || mute_i || (status == STATUS_WAIT && mode & LOOP_ANY))
 			hardStop(frame);
@@ -681,11 +713,11 @@ void SampleChannel::kill(int frame) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::stopBySeq() {
-
+void SampleChannel::stopBySeq()
+{
 	/* kill loop channels and recs if "samplesStopOnSeqHalt" == true,
 	 * else do nothing and return. Always kill at frame=0, this is a
 	 * user-generated event. */
@@ -710,10 +742,11 @@ void SampleChannel::stopBySeq() {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::stop() {
+void SampleChannel::stop()
+{
 	if (mode == SINGLE_PRESS && status == STATUS_PLAY) {
 		if (mute || mute_i)
 			hardStop(0);  /// FIXME - wrong frame value
@@ -726,11 +759,11 @@ void SampleChannel::stop() {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-int SampleChannel::load(const char *file) {
-
+int SampleChannel::load(const char *file)
+{
 	if (strcmp(file, "") == 0 || gIsDir(file)) {
 		gLog("[SampleChannel] file not specified\n");
 		return SAMPLE_LEFT_EMPTY;
@@ -783,11 +816,11 @@ int SampleChannel::load(const char *file) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-int SampleChannel::loadByPatch(const char *f, int i) {
-
+int SampleChannel::loadByPatch(const char *f, int i)
+{
 	int res = load(f);
 
 		volume      = G_Patch.getVol(i);
@@ -806,6 +839,7 @@ int SampleChannel::loadByPatch(const char *f, int i) {
 		readPatchMidiIn(i);
 		midiInReadActions = G_Patch.getMidiValue(i, "InReadActions");
 		midiInPitch       = G_Patch.getMidiValue(i, "InPitch");
+		readPatchMidiOut(i);
 
 	if (res == SAMPLE_LOADED_OK) {
 		setBegin(G_Patch.getBegin(i));
@@ -823,25 +857,27 @@ int SampleChannel::loadByPatch(const char *f, int i) {
 		else
 		if (res == SAMPLE_READ_ERROR)
 			status = STATUS_MISSING;
+		sendMidiLplay();
 	}
 
 	return res;
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-bool SampleChannel::canInputRec() {
+bool SampleChannel::canInputRec()
+	{
 	return wave == NULL;
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::start(int frame, bool doQuantize) {
-
+void SampleChannel::start(int frame, bool doQuantize)
+{
 	switch (status)	{
 		case STATUS_EMPTY:
 		case STATUS_MISSING:
@@ -854,6 +890,7 @@ void SampleChannel::start(int frame, bool doQuantize) {
 		{
 			if (mode & LOOP_ANY) {
 				status = STATUS_WAIT;
+				sendMidiLplay();
 			}
 			else {
 				if (G_Mixer.quantize > 0 && G_Mixer.running && doQuantize)
@@ -864,6 +901,7 @@ void SampleChannel::start(int frame, bool doQuantize) {
 					 * a duplicate call to fillChan occurs with loss of data. */
 
 					status = STATUS_PLAY;
+					sendMidiLplay();
 					if (frame != 0)
 						tracker = fillChan(vChan, tracker, frame);
 				}
@@ -883,31 +921,35 @@ void SampleChannel::start(int frame, bool doQuantize) {
 					reset(frame);
 			}
 			else
-			if (mode & (LOOP_ANY | SINGLE_ENDLESS))
+			if (mode & (LOOP_ANY | SINGLE_ENDLESS)) {
 				status = STATUS_ENDING;
+				sendMidiLplay();
+			}
 			break;
 		}
 
 		case STATUS_WAIT:
 		{
 			status = STATUS_OFF;
+			sendMidiLplay();
 			break;
 		}
 
 		case STATUS_ENDING:
 		{
 			status = STATUS_PLAY;
+			sendMidiLplay();
 			break;
 		}
 	}
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::writePatch(FILE *fp, int i, bool isProject) {
-
+void SampleChannel::writePatch(FILE *fp, int i, bool isProject)
+{
 	Channel::writePatch(fp, i, isProject);
 
 	const char *path = "";
@@ -932,19 +974,20 @@ void SampleChannel::writePatch(FILE *fp, int i, bool isProject) {
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-void SampleChannel::clearChan(float *dest, int start) {
+void SampleChannel::clearChan(float *dest, int start)
+{
 	memset(dest+start, 0, sizeof(float)*(bufferSize-start));
 }
 
 
-/* ------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------- */
 
 
-int SampleChannel::fillChan(float *dest, int start, int offset, bool rewind) {
-
+int SampleChannel::fillChan(float *dest, int start, int offset, bool rewind)
+{
 	int position;  // return value: the new position
 
 	if (pitch == 1.0f) {
